/** 
 *   test_ib_core: IAR Bus (IPC) - Testing implementation (server
 *   side).
 *
 *   (c) Copyright 2010 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>

#include "my_stdio.h"
#include "socket_client.h"
#include "iar_bus.h"
//#include "../version.h"
#define VERSION "UNDEFINED"

#define TEST_IP "127.0.0.1"
#define TEST_PORT 1100

const char *name_definition = {"IAR Bus core"};
const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - verbose mode (incremental).",
        "  -d\t\t - debug mode.",
#       endif
        "  -n, --nodaemon - no run as a daemon.",
        "  -V\t\t - show IAR Bus version.",
        "  --help\t - this help !!!.",
        ""
};

void version(char *program_name)
{
        fprintf(stderr,"%s: ", name_definition);
        fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
        version(program_name);
        fprintf(stderr,
#               ifdef VERBOSE_MODE
 "Usage: \t%s [-vdnh][--nodaemon][--help]\n",
#                else
 "Usage: \t%s [-nh][--nodaemon][--help]\n",
#               endif
                program_name);
}


void help()
{
        int i=0;

        while((char) *help_text[i] != 0)
                fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");
}


int pid_file(char *daemon_name, int pid)
{
        int fd;
        char name[_POSIX_PATH_MAX];
        int ret = -1;

        sprintf(name,"/var/run/%s.pid", daemon_name);
        unlink(name);
        fd = open(name, O_WRONLY|O_CREAT|O_EXCL,
                  S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
        if (fd > 0) {
                fd_printf(fd,"%d\n", pid);
                close(fd);
                ret = 0;
        }

        return ret;
}


int main(int argc, char *argv[]) 
{
	int pid;
	//int i;
	int daemon_flag = 1;
        char *cp;
        char *program_name;
        int op;
        static struct option  const  longopts[] =
        {
                {"help", 0, 0, 'h'},
                {"verbose", 0, 0, 'v'},
                {"nodaemon", 0, 0, 'n'},
                {0,  0,  0,  0}
        };

        /* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];
        opterr =  0;
        while  ((op = getopt_long(argc, argv, "vhnVd",longopts, 0)) != EOF)
                switch( op ){
                case 'v':
                        INCREMENT_VERBOSE_FLAG();
                        break;
#ifdef DEBUG_MODE
                case 'd':
                        debug_flag = 1;
                        daemon_flag = 0;
                        break;
#endif
                case 'n':
                        daemon_flag = 0;
                        break;
                case 'V':
                        version(program_name);
                        exit(-1);
                        break;
                case 'h':
                        usage(program_name);
                        help();
                        exit(-1);
                default:
                        usage(program_name);
                        exit(-1);
                }
        if (daemon_flag)
                pid = fork();
        else
                pid = 0;
	if ( pid != 0 ) {
                pid_file(program_name, pid);
                exit(0);
        }
	printf("Starting IB core...\n");
	//iar_bus_start("test");
	
	iar_bus_set_shutdown(1);
	iar_bus_inet_start(TEST_PORT);

	return 0;
}


